package org.netkernel.lang.beanshell.endpoint;

import java.util.List;

import org.netkernel.container.IKernel;
import org.netkernel.layer0.meta.IEndpointArgumentMeta;
import org.netkernel.layer0.meta.impl.EndpointArgumentMetaImpl;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.util.RequestScopeClassLoader;
import org.netkernel.layer0.util.Utils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.ISpace;
import org.netkernel.lang.beanshell.representation.BeanShellRepresentation;

public class BeanShellPrototype extends StandardAccessorImpl
{
	public BeanShellPrototype()
	{	declareThreadSafe();
		declareTolerantParameters();
	}

public String toString()
	{	return mMeta.getName();
	}
	
	public void onCommissionEndpoint(IKernel aKernel, ISpace aContext) throws Exception
	{	
		Object script = getParameter("script");
		if (script instanceof IIdentifier)
		{	declareName(((IIdentifier)script).toString());
		}
		else
		{	declareName("Beanshell");
		}
		String verbsString = (String)getParameter("verbs");
		int verbs = Utils.parseVerbList(verbsString);
		if (verbs!=0)
		{	declareSupportedVerbs(verbs);
		}
		
		//setup from representations
		INKFParameters params = this.getParameters();
		List<Object> paramValues=params.getParamValues("fromRepresentation");
		Class[] fromRepresentations = new Class[paramValues.size()];
		for (int i=0; i<fromRepresentations.length; i++)
		{	String fromRepString=(String)paramValues.get(i);
			Class fromRep=aContext.getClassLoader().loadClass(fromRepString);
			fromRepresentations[i]=fromRep;
		}
		IEndpointArgumentMeta arg= new EndpointArgumentMetaImpl(IEndpointArgumentMeta.PRIMARY_PART,null,null,null,null,false,fromRepresentations);
		declareArgument(arg);
		
		//setup to representations
		 paramValues=params.getParamValues("toRepresentation");
		for (int i=0; i<paramValues.size(); i++)
		{	String toRepString=(String)paramValues.get(i);
			Class toRep=aContext.getClassLoader().loadClass(toRepString);
			declareSourceRepresentation(toRep);
		}
		
		super.onCommissionEndpoint(aKernel, aContext);
	}
			
	public void onRequest(INKFRequestContext aContext) throws Exception
	{	
		if (aContext.getThisRequest().getVerb()==INKFRequestReadOnly.VERB_META)
		{	aContext.createResponseFrom(mMeta);
		}
		else
		{	BeanShellRepresentation script = (BeanShellRepresentation)aContext.source("param:script", BeanShellRepresentation.class);
			RequestScopeClassLoader cl = new RequestScopeClassLoader(aContext.getKernelContext().getThisKernelRequest().getRequestScope());
			script.execute(aContext,cl);
		}
	}
}
