	/******************************************************************************
	  (c) Copyright 2002-2006, 1060 Research Ltd                                   

	  This Software is licensed to You, the licensee, for use under the terms of   
	  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
	  License v1.0 [www.1060research.com/license] before using or redistributing   
	  this software.                                                               

	  In summary the 1060 Public license has the following conditions.             
	  A. You may use the Software free of charge provided you agree to the terms   
	  laid out in the 1060 Public License v1.0                                     
	  B. You are only permitted to use the Software with components or applications
	  that provide you with OSI Certified Open Source Code [www.opensource.org], or
	  for which licensing has been approved by 1060 Research Limited.              
	  You may write your own software for execution by this Software provided any  
	  distribution of your software with this Software complies with terms set out 
	  in section 2 of the 1060 Public License v1.0                                 
	  C. You may redistribute the Software provided you comply with the terms of   
	  the 1060 Public License v1.0 and that no warranty is implied or given.       
	  D. If you find you are unable to comply with this license you may seek to    
	  obtain an alternative license from 1060 Research Limited by contacting       
	  license@1060research.com or by visiting www.1060research.com                 

	  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
	  LICENSE V1.0 FOR DETAILS                                                     

	  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
	  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

	  File:          $RCSfile$
	  Version:       $Name$ $Revision$
	  Last Modified: $Date$
	 *****************************************************************************/
  package org.netkernel.lang.beanshell.representation;


import org.netkernel.layer0.nkf.*;

import bsh.*;
import java.util.*;

/**
 * Beanshell script engine requiring a main method to aid efficiency
 * @author  tab
 */
public class BeanShellRepresentation
{

	private static boolean sOneshot;
	private final List mInterpreterPool=new ArrayList();
	private String mScript;
	private String mId;
	
	private static final String EX_ID="Unhandled Exception in BeanShellEngine";
	    
	public BeanShellRepresentation(String aScript, String aId, INKFRequestContext aContext) //throws Exception
	{	// create the "global interpreter"- this will exist in a static and cannot be cleaned
		// up so lets not use it and attach it to NK stuff for fear of a memory leak
		if (!sOneshot)
		{	Interpreter i=new Interpreter();
			sOneshot=true;
		}
		mScript=aScript;
		mId=aId;
	}

public  void execute(INKFRequestContext aContext, ClassLoader aClassLoader) throws Exception
	{	Thread.currentThread().setContextClassLoader(aClassLoader);
		Interpreter i = getInterpreter(aContext);
		try
		{	i.set("context", aContext);
			i.eval("main();");
		}
		catch(TargetError e)
		{	Throwable t = e.getTarget();
			if (!(t instanceof Error))
			{	NKFException n=new NKFException(EX_ID, null, mId+" :"+e.getErrorLineNumber()+": "+e.getErrorText(), e.getTarget());
				throw n;
			}
			else
			{	throw (Error)t;
			}
		}
		catch (bsh.EvalError e)
		{	String message = e.getMessage();
			NKFException n=new NKFException(EX_ID, message,mId+" :"+e.getErrorLineNumber()+": "+e.getErrorText(), null );
			throw n;
		}
		finally
		{	i.unset("context");
			release(i);
		}
	}
		
		
	private Interpreter getInterpreter(INKFRequestContext aContext) throws NKFException, EvalError
	{	Interpreter result=null;
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		synchronized(mInterpreterPool)
		{	for (Iterator i=mInterpreterPool.iterator(); i.hasNext(); )
			{	InterpreterStruct is = (InterpreterStruct)i.next();
				synchronized(is)
				{	if (!is.mInUse)
					{	is.mInUse=true;
						result=is.mInterpreter;
						if (!is.mLastClassLoader.equals(cl))
						{	result.setClassLoader(cl);
							is.mLastClassLoader=cl;
						}
						break;
					}
				}
			}
		}
		if (result==null)
		{	InterpreterStruct is = new InterpreterStruct();
			is.mInUse=true;
			is.mLastClassLoader=cl;
			result=new Interpreter();
			is.mInterpreter=result;
			result.setClassLoader(cl);
			try
			{   result.set("context", aContext);
				result.eval(mScript);
				result.unset("context");
				synchronized(mInterpreterPool)
				{	mInterpreterPool.add(is);
				}
			}
			catch(TargetError e)
			{	NKFException n=new NKFException(EX_ID, null, mId+" :"+e.getErrorLineNumber()+": "+e.getMessage() ,e);
				throw n;
			}
			catch(ParseException e)
			{	NKFException n=new NKFException(EX_ID, e.getMessage(), null ,null);
				throw n;
			}
		}
		return result;
	}
		
		private void release(Interpreter aInterpreter)
		{	synchronized(mInterpreterPool)
			{	for (Iterator i=mInterpreterPool.iterator(); i.hasNext(); )
				{	InterpreterStruct is = (InterpreterStruct)i.next();
					if (is.mInterpreter==aInterpreter)
					{	is.mInUse=false;
						break;
					}
				}
			}
		}
		
		public void finalize()
		{	for (Iterator i=mInterpreterPool.iterator(); i.hasNext(); )
			{	InterpreterStruct is = (InterpreterStruct)i.next();
				is.mLastClassLoader=null;
				is.mInterpreter.setClassLoader(null);
				is.mInterpreter=null;
			}
		}
		
		public String toString()
		{	return "BeanShellScript("+mId+")";
		}		
		
		public String getId()
		{	return mId;
		}
		
		private static class InterpreterStruct
		{	public Interpreter mInterpreter;
			public boolean mInUse;
			public ClassLoader mLastClassLoader;
		};	
}	
	
